/*
 * $Id: //depot/prod/newport_news/main/modules/nn-codegen/src/main/java/com/netapp/common/logging/generator/GenerateMsgKeys.java#2 $
 *
 * Copyright (c) 2010 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.logging.generator;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import edu.umd.cs.findbugs.annotations.SuppressWarnings;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;


/**
 * Ant task for generating MsgKey enums from properties files.
 * <p/>
 *
 * A MsgKey enum will be generated for each file in the {@link #propertiesFileSets}.
 * The generated java file will reside in a subdirectory
 * of the {@link #outputDir} that is parallel to the directory
 * containing the properties file.
 * <p/>
 *
 * For example, if the properties file resides (relative to the fileset base dir) at:
 * <pre>
 *   com/netapp/dfm/server/dfm-server-messages.properties
 * </pre>
 * then the resulting MsgKey enum will be generated (relative to the outputDir) at:
 * <pre>
 *   com/netapp/dfm/server/MsgKey.java
 * </pre>
 * and the fully qualified classname will be:
 * <pre>
 *   com.netapp.dfm.server.MsgKey
 * </pre>
 * <p/>
 *
 * The shortname of the enum can be customized by setting the {@link #enumShortName}.
 * The default is "MsgKey".
 * <p/>
 *
 * To use this ant task, declare the taskdef like the following:
 * {@code
 * <pre>
 * <taskdef resource="com/netapp/common/logging/generator/antlib.xml"
 *        uri="antlib:com.netapp.common.logging.generator">
 *     <classpath>
 *         <fileset dir="${nn-codegen.dir}" includes="*.jar"/>
 *     </classpath>
 * </taskdef>
 * </pre>
 * }
 * And then use it like this:
 * {@code
 * <pre>
 * <codegen:generate-msg-keys
 *         enumShortName="MsgKey"
 *         outputDir="${generated.src.main.java.dir}">
 *     <fileset dir="${src.main.resources.dir}">
 *         <include name="&#42;&#42;/&#42;-messages.properties"/>
 *     </fileset>
 * </codegen:generate-msg-keys>
 * </pre>
 * }
 * You'll also need to declare the codegen xml namespace like this:
 * {@code
 * <pre>
 * <project xmlns:codegen="antlib:com.netapp.common.logging.generator">
 * </pre>
 * }
 *
 */
public class GenerateMsgKeys extends Task {

    /**
     * Fileset containing properties files to be used as
     * input for generating the MsgKey enums.
     */
    private List<FileSet> propertiesFileSets = new ArrayList<FileSet>();

    /**
     * Root location into which MsgKey enums will be generated.
     */
    private File outputDir;

    /**
     * The short name (i.e. class name without package name) of the enum to generate.
     * Defaults to "MsgKey".
     */
    private String enumShortName = "MsgKey";

    /**
     * Called by ant to add the given FileSet element to {@link #propertiesFileSets}
     */
    public void addFileset(FileSet buildFiles) {
        this.propertiesFileSets.add(buildFiles);
    }

    public void setOutputDir(File outputDir) {
        this.outputDir = outputDir;
    }

    public void setEnumShortName(String enumShortName) {
        this.enumShortName = enumShortName;
    }


    @Override
    @SuppressWarnings("DP")
    public void execute() {
        if (this.propertiesFileSets.isEmpty()) {
            throw new BuildException(
                    "at least one nested fileset should be provided");
        }
        try {
            for (FileSet propertiesFileSet : this.propertiesFileSets) {
                DirectoryScanner scanner = propertiesFileSet.getDirectoryScanner(getProject());

                List<String> includedFiles = new ArrayList<String>(Arrays.asList(scanner.getIncludedFiles()));

                /*
                 * The bundle is looked up via the class loader,
                 * so use a class loader that is able to find it.
                 */
                URLClassLoader urlClassLoader = new URLClassLoader(new URL[] {scanner.getBasedir().toURI().toURL()}, this.getClass().getClassLoader());

                for (String includedFile : includedFiles) {

                    String enumName = calculateFullEnumName(includedFile);

                    File javaFile = MsgKeyGenerator.calculateJavaFileName(this.outputDir, enumName);

                    if (javaFile.lastModified() >= new File(scanner.getBasedir(), includedFile).lastModified()) {
                        log(String.format("NOT Generating %1$s from %2$s. %1$s is up to date.", enumName, includedFile));
                    } else {
                        log(String.format("Generating %s from %s", enumName, includedFile));

                        MsgKeyGenerator.generateMsgKeyToFile(
                            includedFile,
                            javaFile,
                            enumName,
                            urlClassLoader);
                    }

                }
            }
        } catch (IOException e) {
            throw new BuildException(e);
        }
    }

    private String calculateFullEnumName(String includedFile) {
        String enumName = includedFile;
        if (enumName.endsWith(MsgKeyGenerator.BUNDLE_FILE_SUFFIX)) {
            enumName = enumName.substring(0, enumName.length() - MsgKeyGenerator.BUNDLE_FILE_SUFFIX.length());
        }
        enumName = enumName.replaceAll("(\\\\|/)", ".");

        int lastDot = enumName.lastIndexOf('.');

        String packageName = (lastDot == -1)
            ? enumName
            : enumName.substring(0, lastDot);

        return packageName + "." + this.enumShortName;
    }

}
